<?php

class Easy_SlideIn {

	private static $_instance;

	const NOT_IN_POOL_STATUS = 'esi_not_in_pool';
	const POST_TYPE = 'easy_slide_in';

	private function __construct () {}

	/**
	 * Glues everything together and initialize singleton.
	 */
	public static function init () {
		if (!isset(self::$_instance)) self::$_instance = new self;

		add_action('init', array(self::$_instance, 'register_post_type'));
		add_action('widgets_init', array(self::$_instance, 'register_sidebar'));
		add_action('admin_init', array(self::$_instance, 'add_meta_boxes'));
		add_action('save_post', array(self::$_instance, 'save_meta'), 9); // Bind it a bit earlier, so we can kill Post Indexer actions.
		add_action('wp_insert_post_data', array(self::$_instance, 'set_up_post_status'));

		add_filter("manage_edit-" . self::POST_TYPE . "_columns", array(self::$_instance, "add_custom_columns"));
		add_action("manage_posts_custom_column",  array(self::$_instance, "fill_custom_columns"));
	}

	/**
	 * Prepared singleton object getting routine.
	 */
	public static function get_instance () {
		return self::$_instance;
	}


/* ----- Static info getters ----- */

	/**
	 * Get known themes.
	 */
	public static function get_appearance_themes () {
		return array(
			'minimal' => __('Minimal', 'esi')
		);
	}


	public static function get_default_injection_hook () {
		$hook = defined('ESI_INJECTION_HOOK') ? ESI_INJECTION_HOOK : 'loop_end';
		return apply_filters('esi-core-injection_hook', $hook);
	}


/* ----- Handlers ----- */

	function register_sidebar () {
		$data = new ESI_Options;
		if (!$data->get_option('allow_widgets')) return false;
		register_sidebar(array(
			'name' => __('Easy Slide-In', 'esi'),
			'id' => 'easy-slide-in',
			'description' => __('This sidebar can be used as a Easy Slide-In message content.', 'esi'),
			'class' => '',
			'before_widget' => '<aside id="%1$s" class="esi-widget esi-slide-col %2$s">',
			'after_widget' => '</aside>',
			'before_title' => '<h3 class="esi-widget_title">',
			'after_title' => '</h3>'
		));
	}

	function register_post_type () {
		$supports = apply_filters(
			'esi-slide_in-post_type-supports',
			array('title', 'editor')
		);
		// Force required support
		if (!in_array('title', $supports)) $supports[] = 'title';
		if (!in_array('editor', $supports)) $supports[] = 'editor';

		register_post_type(self::POST_TYPE, array(
			'labels' => array(
				'name' => __('Easy Slide-In', 'esi'),
				'singular_name' => __('Easy Slide-In Message', 'esi'),
				'add_new_item' => __('Add new Easy Slide-In Message', 'esi'),
				'edit_item' => __('Edit Easy Slide-In Message', 'esi'),
			),
			'menu_icon' => ESI_PLUGIN_URL . '/assets/images/admin-menu-icon.png',
			'public' => false,
			'show_ui' => true,
			'supports' => $supports,
		));
		register_post_status(self::NOT_IN_POOL_STATUS, array('protected' => true));
	}

	function add_meta_boxes () {
		add_meta_box(
			'esi_conditions',
			__('Conditions', 'esi'),
			array($this, 'render_conditions_box'),
			self::POST_TYPE,
			'side',
			'high'
		);
		add_meta_box(
			'esi_type',
			__('Easy Slide-In Type', 'esi'),
			array($this, 'render_content_type'),
			self::POST_TYPE,
			'normal',
			'high'
		);
		add_meta_box(
			'esi_show_override',
			__('Easy Slide-In Display Settings Override', 'esi'),
			array($this, 'render_show_after_box'),
			self::POST_TYPE,
			'normal',
			'high'
		);
	}

	function render_conditions_box () {
		global $post;
		$show_options = get_post_meta($post->ID, 'esi_show_if', true);

		echo '<div class="esiui-ui">' .
			'<input type="checkbox" name="not_in_the_pool" id="esi-not_in_the_pool" value="1" ' .
				($post->post_status == self::NOT_IN_POOL_STATUS ? 'checked="checked"' : '') .
			' />' .
			'&nbsp;' .
			'<label for="esi-not_in_the_pool">' . __('Manual Display', 'esi') . '</label>' .
			$this->_create_hint(__('Easy Slide-In posts maked as manual display can be assigned to your individual posts, overriding the defaults that render automatically.', 'esi')) .
		'</div>';

		echo '<div id="esi-conditions-container" class="esiui-ui" style="display:none">';

		echo '<div class="title-div">' . __('Show message if:', 'esi') . '</div>';

		$show_if = esi_getval($show_options, 'user');
		echo '<fieldset id="esi-user_rules"><legend>' . __('User rules', 'esi') . '</legend>';
		echo '' .
			'<input type="radio" name="show_if[user]" value="show_if_logged_in" id="show_if_logged_in-yes" ' .
				('show_if_logged_in' == $show_if ? 'checked="checked"' : '') .
			'/ >' .
			' <label for="show_if_logged_in-yes">' . __('the user is logged in', 'esi') . '</label>' .
		'<br />';
		echo '' .
			'<input type="radio" name="show_if[user]" value="show_if_not_logged_in" id="show_if_not_logged_in-yes" ' .
				('show_if_not_logged_in' == $show_if ? 'checked="checked"' : '') .
			'/ >' .
			' <label for="show_if_not_logged_in-yes">' . __('the user is <b>NOT</b> logged in', 'esi') . '</label> ' .
		'<br />';
		echo '' .
			'<input type="radio" name="show_if[user]" value="show_if_never_commented" id="show_if_never_commented-yes" ' .
				('show_if_never_commented' == $show_if ? 'checked="checked"' : '') .
			'/ >' .
			' <label for="show_if_never_commented-yes">' . __('the user never commented on your site before', 'esi') . '</label> ' .
		'<br />';
		echo '</fieldset>';

		echo '<div class="title-div">' . __('Show message on:', 'esi') . '</div>';

		$show_if = esi_getval($show_options, 'page');
		echo '<fieldset id="esi-page_rules"><legend>' . __('Page rules', 'esi') . '</legend>';
		echo '' .
			'<input type="radio" name="show_if[page]" value="show_if_singular" id="show_if_singular-yes" ' .
				('show_if_singular' == $show_if ? 'checked="checked"' : '') .
			'/ >' .
			' <label for="show_if_singular-yes">' . __('any of my singular pages', 'esi') . '</label>' .
		'<br />';
		echo '' .
			'<input type="radio" name="show_if[page]" value="show_if_not_singular" id="show_if_not_singular-yes" ' .
				('show_if_not_singular' == $show_if ? 'checked="checked"' : '') .
			'/ >' .
			' <label for="show_if_not_singular-yes">' . __('any of my archive pages', 'esi') . '</label>' .
		'<br />';
		echo '' .
			'<input type="radio" name="show_if[page]" value="show_if_home" id="show_if_home-yes" ' .
				('show_if_home' == $show_if ? 'checked="checked"' : '') .
			'/ >' .
			' <label for="show_if_home-yes">' . __('home page', 'esi') . '</label>' .
		'<br />';
		$types = get_post_types(array('public' => true), 'objects');
		foreach ($types as $type => $obj) {
			if ('attachment' == $type) continue;
			$name = "show_if_{$type}";
			echo '' .
				'<input type="radio" name="show_if[page]" value="' . $name . '" id="' . $name . '-yes" ' .
					($name == $show_if ? 'checked="checked"' : '') .
				'/ >' .
				' <label for="' . $name . '-yes">' . sprintf(__('&quot;%s&quot; post pages', 'esi'), $obj->label) . '</label>' .
			'<br />';
		}

		echo '</fieldset>';

		echo '</div>';
	}

	function render_content_type () {
		global $post;
		$opts = get_post_meta($post->ID, 'esi-type', true);
		$type = esi_getval($opts, 'content_type', 'text');

		echo '<div class="esiui-ui esiui-meta">';

		echo '' .
			'<input type="radio" name="esi-type[content_type]" id="esi-content_type-text" value="text" ' . ('text' == $type ? 'checked="checked"' : '') . ' />' .
			'&nbsp;' .
			'<label for="esi-content_type-text">' . __('Text message', 'esi') . '</label>' .
		'<br />';
		echo '' .
			'<input type="radio" name="esi-type[content_type]" id="esi-content_type-mailchimp" value="mailchimp" ' . ('mailchimp' == $type ? 'checked="checked"' : '') . ' />' .
			'&nbsp;' .
			'<label for="esi-content_type-mailchimp">' . __('MailChimp subscription form', 'esi') . '</label>' .
		'<br />';
		echo '' .
			'<input type="radio" name="esi-type[content_type]" id="esi-content_type-related" value="related" ' . ('related' == $type ? 'checked="checked"' : '') . ' />' .
			'&nbsp;' .
			'<label for="esi-content_type-related">' . __('Related posts', 'esi') . '</label>' .
		'<br />';
		$data = new ESI_Options;
		if ($data->get_option('allow_widgets')) {
			echo '' .
				'<input type="radio" name="esi-type[content_type]" id="esi-content_type-widgets" value="widgets" ' . ('widgets' == $type ? 'checked="checked"' : '') . ' />' .
				'&nbsp;' .
				'<label for="esi-content_type-widgets">' . __('Sidebar widgets', 'esi') . '</label>' .
			'<br />';
		}

		// --- Message
		echo '<div id="esi-content_type-options-text" class="esi-content_type" style="display:none"></div>';

		// --- MailChimp
		echo '<div id="esi-content_type-options-mailchimp" class="esi-content_type" style="display:none">';
		echo '<table border="0" cellpadding="2" cellspacing="0" width="100%" class="esiui-table">';
		echo '<col width="25%"/><col width="75%"/>';
		echo '<tr>';
		$defaults = get_option('esi');
		$api_key = esi_getval($opts, 'mailchimp-api_key', esi_getval($defaults, 'mailchimp-api_key'));
		echo '<td class="param"><label for="esi-mailchimp-api_key">' . __('MailChimp API key:') . '</label></td>' .
			'<td><input type="text" class="long" name="esi-type[mailchimp-api_key]" id="esi-mailchimp-api_key" value="' . esc_attr($api_key) . '" /></td>' .
		'</tr>';
		if (!$api_key) {
			echo '<tr><td></td><td>';
			echo $this->_create_hint(__('Enter your API key here, then save the post to continue', 'esi'));
			echo '</td></tr>';
		} else {
			$mailchimp = new ESI_Mailchimp($api_key);

			$lists = $mailchimp->get_lists();
			$current = esi_getval($opts, 'mailchimp-default_list', esi_getval($defaults, 'mailchimp-default_list'));
			echo '<tr>';
			echo '<td class="param"><label>' . __('Default subscription list:', 'esi') . ' </label></td>';
			echo '<td><div class="esiui-ui-select"><select name="esi-type[mailchimp-default_list]">';
			echo '<option></option>';
			foreach ($lists as $list) {
				$selected = $list['id'] == $current ? 'selected="selected"' : '';
				echo '<option value="' . esc_attr($list['id']) . '" ' . $selected . '>' . $list['name'] . '</option>';
			}
			echo '</select></div></td>';
			echo '</tr>';
			// We got this far, we have the API key
			//echo '&nbsp;<a href="#mcls-refresh" id="esi-mcls-refresh">' . __('Refresh', 'esi') . '</a>';
			//echo $this->_create_hint(__('Select a default list you wish to subscribe your visitors to.', 'esi'));

			$placeholder = esi_getval($opts, 'mailchimp-placeholder', 'you@yourdomain.com');
			echo '<tr>';
			echo '<td class="param"><label for="esi-mailchimp-placeholder">' . __('Placeholder text:', 'esi') . '</label></td>' .
				'<td><input type="text" class="long" name="esi-type[mailchimp-placeholder]" id="esi-mailchimp-placeholder" value="' . esc_attr($placeholder) . '" />' .
			'</td>';
			echo '</tr>';
			$position = esi_getval($opts, 'mailchimp-position', 'after');
			echo '<tr>';
			echo '<td class="param"><label for="esi-mailchimp-position-after">' . __('Show my form:', 'esi') . '</label></td>';
			echo '<td>' .
				'<input type="radio" name="esi-type[mailchimp-position]" id="esi-mailchimp-position-after" value="after" ' . checked('after', $position, false) . ' />' .
				'<label for="esi-mailchimp-position-after">' . __('After the message text', 'esi') . '</label>' .
			'<br />';
			echo '' .
				'<input type="radio" name="esi-type[mailchimp-position]" id="esi-mailchimp-position-before" value="before" ' . checked('before', $position, false) . ' />' .
				'<label for="esi-mailchimp-position-before">' . __('Before the message text', 'esi') . '</label>' .
			'</td>';
			echo '</tr>';

			$subscription_message = esi_getval($opts, 'mailchimp-subscription_message', esi_getval($defaults, 'mailchimp-subscription_message'));
			$subscription_message = $subscription_message ? $subscription_message : __('All good, thank you!', 'esi');
			$subscription_message = wp_strip_all_tags($subscription_message);
			echo '<tr>';
			echo '<td class="param">' .
				'<label for="esi-mailchimp-subscription_message">' . __('Successful subscription message:', 'esi') . '</label></td>' .
				'<td><input type="text" class="long" name="esi-type[mailchimp-subscription_message]" id="esi-mailchimp-subscription_message" value="' . esc_attr($subscription_message) . '" /></td>' .
			'</tr>';
		}
		echo '</table>';
		echo '</div>';

		// --- Related posts
		echo '<div id="esi-content_type-options-related" class="esi-content_type" style="display:none">';
		echo '<table border="0" cellpadding="2" cellspacing="0" width="100%" class="esiui-table">';
		echo '<col width="25%"/><col width="75%"/>';
		$count = esi_getval($opts, 'related-posts_count', 3);
		echo '<tr>';
		echo '<td class="param"><label>' . __('Show this many related posts:', 'esi') . ' </label></td>';
		echo '<td><div class="esiui-ui-select"><select name="esi-type[related-posts_count]">';
		foreach (range(1, 10) as $item) {
			$selected = $item == $count ? 'selected="selected"' : '';
			echo '<option value="' . esc_attr($item) . '" ' . $selected . '>' . $item . '</option>';
		}
		echo '</select></div></td>';
		echo '</tr>';

		$taxonomies = get_taxonomies(array(
			'public' => true,
		), 'objects');
		$related_tax = esi_getval($opts, 'related-taxonomy', 'post_tag');
		
		echo '<tr>';
		echo '<td class="param"><label>' . __('Related taxonomy:', 'esi') . ' </label></td>';
		echo '<td><div class="esiui-ui-select"><select name="esi-type[related-taxonomy]">';
		foreach ($taxonomies as $tax => $item) {
			$selected = $tax == $related_tax ? 'selected="selected"' : '';
			echo '<option value="' . esc_attr($tax) . '" ' . $selected . '>' . $item->label . '</option>';
		}
		echo '</select></div></td>';
		echo '</tr>';
		$has_thumbnails = esi_getval($opts, 'related-has_thumbnails');
		echo '<td class="param">' . '<label for="esi-has_thumbnails">' . __('Show thumbnails?', 'esi') . '</label></td>' .
			'<td><input type="hidden" name="esi-type[related-has_thumbnails]" value="" />' .
			'<input type="checkbox" id="esi-has_thumbnails" name="esi-type[related-has_thumbnails]" value="1" ' . ($has_thumbnails ? 'checked="checked"' : '') . ' />' .
			'</td>' .
			 
		'</tr></table>';
		echo '</div>';

		// --- Widgets
		if ($data->get_option('allow_widgets')) {
			echo '<div id="esi-content_type-options-widgets" class="esi-content_type" style="display:none"></div>';
		}

		echo '</div>';
	}

	function render_show_after_box () {
		global $post;
		$opts = get_post_meta($post->ID, 'esi', true);
		$condition = esi_getval($opts, 'show_after-condition');
		$value = esi_getval($opts, 'show_after-rule');
		$selector = "";
		$timeout = "";
		$percentage = "";
		
		switch ($condition) {
			case "selector":
				$selector = 'checked="checked"';
				break;
			case "timeout":
				$timeout = 'checked="checked"';
				$value = (int)$value;
				break;
			case "percentage":
				$percentage = 'checked="checked"';
				$value = (int)$value;
				break;
		}

		$pos = esi_getval($opts, 'position');
		$width = esi_getval($opts, 'width');
		
		$override_checked = ($percentage || $timeout || $selector || $pos || $width) ? 'checked="checked"' : '';
		echo '<div class="esi-override-display">';
		echo '<p class="esiui-ui">' .
			'<input type="checkbox" id="esi-override_show_if" name="esi-appearance_override" value="1" ' . $override_checked . ' /> ' .
			'<label for="esi-override_show_if">' . __('Override Global Settings', 'esi') . '</label>' .
		'</p>';

		echo '<div id="esi-show_after_overrides-container" class="esiui-ui" style="display:none">';
		echo '<div class="title-div">Activate display settings</div>';
		// Initial condition
		echo '<fieldset id="esi-show_after"><legend>' . __('Show after', 'esi') . '</legend>';

		$percentage_select = '<div class="esiui-ui-select"><select name="esi[show_after-rule]" ' . ($percentage ? '' : 'disabled="disabled"') . '>';
		for ($i=1; $i<100; $i++) {
			$selected = ($i == $value) ? 'selected="selected"' : '';
			$percentage_select .= "<option value='{$i}' {$selected}>{$i}&nbsp;</option>";
		}
		$percentage_select .= '</select></div>%';
		echo '<div>' .
			'<input type="radio" name="esi[show_after-condition]" value="percentage" id="esi-show_after-percentage" ' . $percentage . ' /> ' .
			'<label for="esi-show_after-percentage">' .
				__('Show message after this much of my page has been viewed', 'esi') .
				': ' .
			'</label>' .
			$percentage_select .
		'</div>';

		echo '<div>' .
			'<input type="radio" name="esi[show_after-condition]" value="selector" id="esi-show_after-selector" ' . $selector . ' /> ' .
			'<label for="esi-show_after-selector">' .
				__('Show message after scrolling past element with this ID', 'esi') .
				': #' .
			'</label>' .
			'<input type="text" size="8" name="esi[show_after-rule]" id="" value="' . ($selector ? esc_attr($value) : '') . '" ' . ($selector ? '' : 'disabled="disabled"') . ' />' .
		'</div>';

		echo '<div>' .
			'<input type="radio" name="esi[show_after-condition]" value="timeout" id="esi-show_after-timeout" ' . $timeout . ' /> ' .
			'<label for="esi-show_after-timeout">' .
				__('Show message after this many seconds', 'esi') .
				': ' .
			'</label>' .
			'<input type="text" size="2" name="esi[show_after-rule]" id="" value="' . ($timeout ? esc_attr($value) : '') . '" ' . ($timeout ? '' : 'disabled="disabled"') . ' />' .
		'</div>';
		echo '</fieldset>';

		// Timeout
		echo '<fieldset id="esi-show_for"><legend>' . __('Show for', 'esi') . '</legend>';
		$time = esi_getval($opts, 'show_for-time');
		$unit = esi_getval($opts, 'show_for-unit');

		$_times = array_combine(range(1,59), range(1,59));
		$_units = array(
			's' => __('Seconds', 'esi'),
			'm' => __('Minutes', 'esi'),
			'h' => __('Hours', 'esi'),
		);

		echo "<div class='esiui-ui-select'><select name='esi[show_for-time]'>";
		foreach ($_times as $_time) {
			$selected = $_time == $time ? 'selected="selected"' : '';
			echo "<option value='{$_time}' {$selected}>{$_time}</option>";
		}
		echo "</select></div>";

		echo "<div class='esiui-ui-select'><select name='esi[show_for-unit]'>";
		foreach ($_units as $key => $_unit) {
			$selected = $key == $unit ? 'selected="selected"' : '';
			echo "<option value='{$key}' {$selected}>{$_unit}</option>";
		}
		echo "</select></div>";
		echo '</fieldset>';
		echo '<div class="title-div">Position settings</div>';
		
		// Position
		echo '<fieldset id="esi-position"><legend>' . __('Position', 'esi') . '</legend>';
		echo '<div  class="esiui-ui-element_container">';
		echo '<div class="position-control">' .
			$this->_create_radiobox('position', 'left', $pos) .
			$this->_create_radiobox('position', 'top', $pos) .
			$this->_create_radiobox('position', 'right', $pos) .
			$this->_create_radiobox('position', 'bottom', $pos) .
			$this->_create_radiobox('position', 'bottom_left', $pos) .
			$this->_create_radiobox('position', 'bottom_right', $pos) .
			$this->_create_radiobox('position', 'top_left', $pos) .
			$this->_create_radiobox('position', 'top_right', $pos) .
			$this->_create_radiobox('position', 'center', $pos) .
			'</div>';
		echo '</div>';
		echo '</fieldset>';
		echo '<div class="title-div">Width settings</div>';
		echo '<fieldset id="esi-widthsettings"><legend>' . __('Width', 'esi') . '</legend>';
		
		$checked = (!(int)$width || 'full' == 'width') ? 'checked="checked"' : '';
		echo '' .
			'<input type="checkbox" name="esi[width]" value="full" id="esi-full_width" ' . $checked . ' autocomplete="off" />' .
			'&nbsp;' .
			'<label for="esi-full_width">' . __('Full width', 'esi') . '</label>' .
		'';
		$display = $checked ? 'style="display:none"' : '';
		echo '<div id="esi-custom_width" ' . $display . '>';
		$disabled = $checked ? 'disabled="disabled"' : '';
		echo '' .
			'<label for="esi-width">' . __('Message width', 'esi') . '</label>' .
			'&nbsp;' .
			'<input type="text" size="8" class="medium" name="esi[width]" id="esi-width" value="' . (int)$width . '" ' . $disabled . ' />px' .
		'';
		echo '</div>';
		
		echo '<br />';
		$vetical_cols = esi_getval($opts, 'vetical_cols');
		$checked = ($vetical_cols == "yes") ? 'checked="checked"' : '';
		echo '' .
				'<input type="checkbox" name="esi[vetical_cols]" value="yes" id="esi-vetical_cols" ' . $checked . ' autocomplete="off" />' .
				'&nbsp;' .
				'<label for="esi-vetical_cols">' . __('Display content vertically', 'esi') . '</label>' .
				'';
		
		echo '</fieldset>';

		echo '<div class="title-div">Style settings</div>';
		echo '<fieldset id="esi-widthsettings"><legend>' . __('Colors', 'esi') . '</legend>';
		$bgcolor = esi_getval($opts, 'bgcolor');
		$textcolor =esi_getval($opts, 'textcolor'); 
		$accentcolor =esi_getval($opts, 'accentcolor'); 
		
		echo '<table border="0" celpadding="5" cellspacing="0" width="100%"><col width="15%"/><col width="85%"/><tr><td colspan="2">' .
				$this->_create_hint(__('Customize default main colors of slide in panel. More detailed customization can be done with additional CSS field', 'esi')). '';
		echo '</td></tr><tr><td><span for="esi-bgcolor">' . __('Background color:') . '</span></td>' .
				'<td><input type="text" class="long" name="esi[bgcolor]" id="esi-bgcolor" value="' . esc_attr($bgcolor) . '" />' .
				'</td></tr>';
		echo '<tr><td><span for="esi-textcolor">' . __('Text color:') . '</span></td>' .
				'<td><input type="text" class="long" name="esi[textcolor]" id="esi-textcolor" value="' . esc_attr($textcolor) . '" />' .
				'</td></tr>';
		echo '<tr><td><span for="esi-accentcolor">' . __('Other color:') . '</span></td>' .
				'<td><input type="text" class="long" name="esi[accentcolor]" id="esi-accentcolor" value="' . esc_attr($accentcolor) . '" />' .
				'</td></tr></table>';
		
		echo '</fieldset>';
		echo '</div></div>';
		
		echo '
<script type="text/javascript">


jQuery(document).ready(function($){
    $(\'#esi-bgcolor\').wpColorPicker();
    $(\'#esi-textcolor\').wpColorPicker();
    $(\'#esi-accentcolor\').wpColorPicker();
    
});

</script>';
	}


	/**
	 * Saves metabox data.
	 */
	function save_meta () {
		global $post;
		if ($post && self::POST_TYPE != $post->post_type) return false;
		if (esi_getval($_POST, 'show_if')) {
			update_post_meta($post->ID, "esi_show_if", esi_getval($_POST, "show_if"));
		}
		else {
			update_post_meta($post->ID, "esi_show_if", false);
		}

		if (esi_getval($_POST, 'esi')) {
			// If we have Post Indexer present, remove the post save action for the moment.

			if (!empty($_POST['esi-appearance_override'])) update_post_meta($post->ID, "esi", esi_getval($_POST, "esi"));
			else update_post_meta($post->ID, "esi", false);

		}
		if (!empty($_POST['esi-type']['content_type'])) update_post_meta($post->ID, "esi-type", esi_getval($_POST, "esi-type"));
	}

	/**
	 * Updates pool status.
	 */
	function set_up_post_status ($data) {
		if (self::POST_TYPE != $data['post_type']) return $data;
		if (esi_getval($_POST, 'not_in_the_pool')) {
			$data['post_status'] = self::NOT_IN_POOL_STATUS;
		}
		return $data;
	}

	function add_custom_columns ($cols) {
		return array_merge($cols, array(
			'esi_type' => __('Content Type', 'esi'),
			'esi_pool' => __('Status', 'esi'),
			'esi_conditions' => __('Conditions', 'esi'),
		));
	}

	function fill_custom_columns ($col) {
		global $post;
		if ('esi_pool' != $col && 'esi_conditions' != $col && 'esi_type' != $col) return $col;

		switch ($col) {
			case 'esi_pool':
				echo ('publish' == $post->post_status ? __('Automatic Display', 'esi') : __('Manual Display', 'esi'));
				break;
			case 'esi_type':
				$all_content_types = array(
					'text' => __('Text message', 'esi'),
					'mailchimp' => __('MailChimp subscription form', 'esi'),
					'related' => __('Related posts', 'esi'),
					'widgets' => __('Sidebar widgets', 'esi'),
				);
				$type = get_post_meta($post->ID, 'esi-type', true);
				$content_type = esi_getval($type, 'content_type', 'text');
				if (!empty($all_content_types[$content_type])) echo $all_content_types[$content_type];
				break;
			case 'esi_conditions':
				if (self::NOT_IN_POOL_STATUS == $post->post_status) {
					$post_links = array();
					global $wpdb;
					$appears_on = $wpdb->get_col($wpdb->prepare("SELECT post_id FROM {$wpdb->postmeta} WHERE (meta_key='esi_message_id' OR meta_key='esi_message_id2' OR meta_key='esi_message_id3' OR meta_key='esi_message_id4' OR meta_key='esi_message_id5') AND meta_value=%d", $post->ID));
					if (empty($appears_on)) {
						_e("Not applicable", 'esi');
						break;
					} else foreach ($appears_on as $target_id) {
						$post_links[] = '<a href="' . admin_url('post.php?action=edit&post=' . $target_id) . '">' . get_the_title($target_id) . '</a>';
					}
					printf(__('Appears on %s', 'esi'), join('<br />', $post_links));
					break;
				}
				$show = get_post_meta($post->ID, 'esi_show_if', true);
				switch (esi_getval($show, 'user')) {
					case "show_if_logged_in":
						_e("Shown for logged in users", 'esi');
						break;
					case "show_if_not_logged_in":
						_e("Shown for visitors", 'esi');
						break;
					case "show_if_never_commented":
						_e("Shown for non-commenters", 'esi');
						break;
					default:
						_e("Can appear for all users", 'esi');
				}
				echo '<br />';
				$types = get_post_types(array('public' => true), 'objects');
				$show_page = esi_getval($show, 'page');
				switch ($show_page) {
					case "show_if_singular":
						_e("Shown on singular pages", 'esi');
						break;
					case "show_if_not_singular":
						_e("Shown on archive pages", 'esi');
						break;
					case "show_if_home":
						_e("Shown on home page", 'esi');
						break;
					default:
						$shown_for_types = array();
						foreach ($types as $type => $obj) {
							if ("show_if_{$type}" == $show_page) $shown_for_types[] = sprintf(__('Shown on %s pages', 'esi'), $obj->labels->name);
						}
						if ($shown_for_types) {
							echo join('<br />', $shown_for_types);
						} else {
							_e("Can appear on all pages", 'esi');
						}
						break;
				}
				break;
		}
	}


/* ----- Model procedures: message ----- */


	public function get_message_data ($post) {
		$post_id = (is_object($post) && isset($post->ID)) ? $post->ID : (int)$post_id;

		// ...

		//$post_id = 2852;
		$pool = $this->_get_active_messages_pool($post_id);
		
		$display_all_manual = get_post_meta($post_id, 'esi_display_all', true);
		if ($display_all_manual) {
			return $pool ? $pool : false;
		}
		else {
			return $pool ? array($pool[0]) : false;
		}
	}


/* ----- Model procedures: pool ----- */


	/**
	 * Fetching out all the currently active messages.
	 */
	private function _get_active_messages_pool ($specific_post_id=false) {
		$pool = array();
		$query = new WP_Query(array(
			'post_type' => self::POST_TYPE,
			'posts_per_page' => -1,
		));
		$pool = $query->posts ? $query->posts : array();

		if ($specific_post_id) {
			$msg_id = get_post_meta($specific_post_id, 'esi_message_id', true);
			if ($msg_id) $pool = array(get_post($msg_id));

			$msg_id2 = get_post_meta($specific_post_id, 'esi_message_id2', true);
			if ($msg_id2) $pool[] = (get_post($msg_id2));

			$msg_id3 = get_post_meta($specific_post_id, 'esi_message_id3', true);
			if ($msg_id3) $pool[] = (get_post($msg_id3));

			$msg_id4 = get_post_meta($specific_post_id, 'esi_message_id4', true);
			if ($msg_id4) $pool[] = (get_post($msg_id4));

			$msg_id5 = get_post_meta($specific_post_id, 'esi_message_id5', true);
			if ($msg_id5) $pool[] = (get_post($msg_id5));
				
		}
		$pool = array_filter($pool, array($this, '_filter_active_messages_pool'));
		shuffle($pool);

		return $pool;
	}

	/**
	 * Filters messages in pool to active ones.
	 * `array_filter` callback.
	 */
	function _filter_active_messages_pool ($msg) {
		$use = true;
		$show = get_post_meta($msg->ID, 'esi_show_if', true);
		switch (esi_getval($show, 'user')) {
			case "show_if_logged_in":
				$use = is_user_logged_in(); break;
			case "show_if_not_logged_in":
				$use = !(is_user_logged_in()); break;
			case "show_if_never_commented":
				$use = isset($_COOKIE['comment_author_'.COOKIEHASH]); break;
		}
		if (!$use) return $use;

		$page_condition = esi_getval($show, 'page');
		if (empty($page_condition)) return true;

		switch ($page_condition) {
			case "show_if_singular":
				$use = is_singular(); break;
			case "show_if_not_singular":
				$use = !(is_singular()); break;
			case "show_if_home":
				$use = is_front_page(); break;
		}
		if (!$use) return $use;
		$types = get_post_types(array('public' => true), 'objects');
		foreach ($types as $type => $obj) {
			$name = "show_if_{$type}";
			if ($name == $page_condition) $use = is_singular($type);
		}
		return $use; // In the pool, by default
	}

	function _create_radiobox ($name, $value, $option) {
		$checked = (@$option == $value) ? true : false;
		$class = $value ? "class='{$value}'" : '';
		return "<input type='radio' name='esi[{$name}]' {$class} id='{$name}-{$value}' value='{$value}' " . ($checked ? 'checked="checked" ' : '') . " /> ";
	}

	function _create_color_radiobox ($name, $value, $label, $option) {
		$color = esc_attr($value);
		$label= esc_attr($label);
		return "<label class='esi-color-container' for='{$name}-{$value}'>" .
			$this->_create_radiobox($name, $value, $option) .
			"<div class='esi-color esi-{$color}' title='{$label}'></div>" .
		'</label>';
	}

	function _create_hint ($text) {
		return "<p class='info'><span class='info'></span>{$text}</p>";
	}
}